@@ -1,7 +1,9 @@
 #ifndef WCALC_ISFUNC
 #define WCALC_ISFUNC
 
-int isfunc(const char *str);
+#include "explain.h"
 
+extern const struct name_with_exp funcs[];
+int isfunc(const char *str);
 #endif
 /* vim:set expandtab: */
