@@ -8,6 +8,7 @@
 #endif
 #include <stdio.h>                     /* for snprintf() */
 #include <stdlib.h>                    /* for calloc() */
+#include <sys/types.h>                 /* for ssize_t */
 
 /* Internal Headers */
 #include "number.h"
@@ -95,7 +96,8 @@ char *num_to_str_complex(const Number           num,
             significant_figures = ((e > 0) ? e : 0) + prec;
             Dprintf("sig figs = %zu\n", significant_figures);
             if (significant_figures < 2) {      /* MPFR-defined minimum (why?) */
-                s = num_get_str(s, &e, base, 2, num);
+                num_free_str(s);
+                s = num_get_str(NULL, &e, base, 2, num);
                 Dprintf("s=%s, e = %i\n", s, (int)e);
                 if (s[1] > '4') {      /* XXX: LAME! */
                     unsigned foo;
@@ -127,6 +129,7 @@ char *num_to_str_complex(const Number           num,
             }
             Dprintf("left_digits = %i, asking for %i\n", left_digits,
                     left_digits + prec);
+            num_free_str(s);
             s = num_get_str(NULL, &e, base, ((left_digits + prec) < 2) ? 2 : (left_digits + prec), num);
         }
     }
@@ -313,7 +316,7 @@ char *full_precision_formatted_number(const char *digits,
     size_t length;
     size_t full_length;
     size_t decimal_count = 0;
-    size_t printed;
+    //size_t printed;
     char  *retstring, *curs;
     size_t d_index = 0;
 
@@ -380,10 +383,10 @@ char *full_precision_formatted_number(const char *digits,
 
     // this variable exists because snprintf's return value is unreliable.
     // and can be larger than the number of digits printed
-    printed = ((length - 1 < strlen(&digits[d_index])) ? length - 1 : strlen(&digits[d_index]));
+    //printed = ((length - 1 < strlen(&digits[d_index])) ? length - 1 : strlen(&digits[d_index]));
     snprintf(curs, length, "%s", &digits[d_index]);
-    length        -= printed;
-    decimal_count += printed;
+    //length        -= printed;
+    //decimal_count += printed;
 
     // strip off the trailing 0's
     zero_strip(retstring);
@@ -409,7 +412,7 @@ char *automatically_formatted_number(const char *digits,
     size_t     length;
     size_t     full_length;
     size_t     decimal_count = 0;
-    size_t     printed;
+    //size_t     printed;
     char      *retstring, *curs;
     size_t     d_index      = 0;
     const long original_exp = exp - 1;
@@ -479,10 +482,10 @@ char *automatically_formatted_number(const char *digits,
 
     // this variable exists because snprintf's return value is unreliable.
     // and can be larger than the number of digits printed
-    printed = ((length - 1 < strlen(digits + d_index)) ? length - 1 : strlen(digits + d_index));
+    //printed = ((length - 1 < strlen(digits + d_index)) ? length - 1 : strlen(digits + d_index));
     snprintf(curs, length, "%s", digits + d_index);
-    length        -= printed;
-    decimal_count += printed;
+    //length        -= printed;
+    //decimal_count += printed;
 
     if (precision == -1) {
         char *period;
